#!/usr/bin/bash

SKIP_BOOT_SIZE=4
BOOT_SIZE=32
MODULE_SIZE=32
ROOTFS_SIZE=2048
DATA_SIZE=1024
USER_SIZE=""

function get_size {
	OLD_SFDISK=false

	SIZE=`  sfdisk -s /dev/mmcblk0`
	test "$SIZE" != "" || die "The disk name (/dev/mmcblk0) is incorrect. Please, enter valid disk name.  /dev/sd[x]"

	SDCARD_SIZE=$((SIZE >> 10))
	#USER_SIZE=500
	USER_SIZE=`expr $SDCARD_SIZE - $SKIP_BOOT_SIZE - $BOOT_SIZE - $MODULE_SIZE - $ROOTFS_SIZE - $DATA_SIZE - 2`
	test 100 -lt $USER_SIZE || echo "We recommend to use more than 4GB disk" && USER_SIZE=500
}


function repartition_sd {
	local BOOT=boot
	local MODULE=modules
	local ROOTFS=rootfs
	local SYSTEMDATA=system-data
	local USER=user

	umount /dev/mmcblk0*

	echo "Remove partition table..."                                                
	su -c "dd if=/dev/zero of=/dev/mmcblk0 bs=512 count=16 conv=notrunc"



sfdisk /dev/mmcblk0 <<-__EOF__
4MiB,32MiB,0xE,*
8MiB,32MiB,,-
8MiB,2048MiB,,-
8MiB,,E,-
,1024MiB,,-
,${USER_SIZE}MiB,,-
__EOF__


	  mkfs.ext4 -q /dev/mmcblk0p1 -L $BOOT -F
	  mkfs.ext4 -q /dev/mmcblk0p2 -L $MODULE -F
	  mkfs.ext4 -q /dev/mmcblk0p3 -L $ROOTFS -F
	  mkfs.ext4 -q /dev/mmcblk0p5 -L $SYSTEMDATA -F
	  mkfs.ext4 -q /dev/mmcblk0p6 -L $USER -F
}


function copy_boot_files {
	dd if=/etc/emmcboot_files/bl1-emmcboot.img of=/dev/mmcblk0 bs=512 seek=1
	dd if=/etc/emmcboot_files/loader-emmc.img of=/dev/mmcblk0 bs=512 seek=129
	dd if=/etc/emmcboot_files/bl_mon.img of=/dev/mmcblk0 bs=512 seek=513
	dd if=/etc/emmcboot_files/secureos.img of=/dev/mmcblk0 bs=512 seek=769
	dd if=/etc/emmcboot_files/bootloader.img of=/dev/mmcblk0 bs=512 seek=3841
	dd if=/etc/emmcboot_files/params.bin of=/dev/mmcblk0 bs=512 seek=5889
}


function copy_partition {
	echo "Copying partition $1"

	cd /

	#mount /dev/mmcblk1p$1 from_mount
	#mount /dev/mmcblk0p$1 to_mount
	
	#cp -r from_mount/* to_mount/

	#/etc/emmcboot_files/rsync -ax from_mount/* to_mount/

	dd if=/dev/mmcblk1p$1 of=/dev/mmcblk0p$1 bs=2048	
	sync;sync;

	#umount from_mount
	#umount to_mount
}


function copy_root_partition {
	echo "Copying partition $1"

	cd /

	#mount /dev/mmcblk1p$1 from_mount
	#mount /dev/mmcblk0p$1 to_mount
	
	#cp -r from_mount/* to_mount/

	#/etc/emmcboot_files/rsync -ax from_mount/* to_mount/ --exclude={/dev/*,/proc/*,/sys/*,/tmp/*}

	# dd if=/etc/emmcboot_files/rootfs.img of=/dev/mmcblk0p$1 bs=2048	
	dd if=/dev/mmcblk1p7 of=/dev/mmcblk0p$1 bs=2048
	sync;sync;

	#umount from_mount
	#umount to_mount
}


export PATH=$PATH:/usr/sbin

#init

umount /dev/mmcblk1p1
umount /dev/mmcblk1p2
umount /dev/mmcblk1p5
umount /dev/mmcblk1p6

cd /

get_size

repartition_sd

mkdir to_mount
mkdir from_mount

copy_boot_files

copy_partition 1
copy_partition 2
copy_root_partition 3
copy_partition 5
copy_partition 6

mount /dev/mmcblk0p1 from_mount
mount from_mount/ramdisk.img to_mount

cp /etc/emmcboot_files/init to_mount/sbin/init
chmod +x to_mount/sbin/init
sync;sync;

umount to_mount
umount from_mount

rmdir to_mount
rmdir from_mount

poweroff
